import { scaleLinear } from "d3-scale";
import { format } from "d3-format";
import { DEFAULT_TICKS, DEFAULT_TICK_FORMAT } from "./constants";
/** handles configuring the x-scale and x-axis for non-categorical legends
 * @ignore - for custom-elements.json
 */
export class AxisTicksSetter {
    constructor(cle) {
        this.cle = cle;
    }
    /**
     * Sets the xScale property based on the value of the scaleType property
     */
    setXScale() {
        const { scaleType, marginLeft, width, marginRight } = this.cle;
        switch (scaleType) {
            case "continuous":
                this.xScale = scaleLinear()
                    .domain(this.cle.domain)
                    .range([marginLeft, width - marginRight]);
                break;
            case "discrete":
            case "threshold":
                this.xScale = scaleLinear()
                    .domain([
                    this.cle.domain[0],
                    this.cle.domain[this.cle.domain.length - 1],
                ])
                    .rangeRound([marginLeft, width - marginRight]);
                break;
            case "categorical":
                // xScale is not used for ScaleType.Categorical
                this.xScale = null;
                break;
            default:
                throw new Error(`Unrecognized scaleType: ${scaleType}`);
        }
    }
    /**
     * Handles setting fallback axis tick values for discrete & threshold scales
     * Handles setting the tickFormatter function
     */
    handleAxisTicks() {
        if ((this.cle.scaleType === "discrete" ||
            this.cle.scaleType === "threshold") &&
            !this.cle.tickValues) {
            const [min, max] = this.xScale.domain();
            this.cle.tickValues = [
                min,
                ...(this.cle.colorScale?.thresholds?.() ||
                    this.cle.colorScale.domain()),
                max,
            ];
        }
        // prefer `tickFormatter` property if it has been set
        // TODO: how to override `tickFormat` after tickFormatter has previously been set and is already a function?
        if (typeof this.cle.tickFormatter === "function") {
            return;
        }
        else if (this.cle.tickFormat?.length) {
            // else prefer `tickFormat` attribute / property if it has been set
            this.cle.tickFormatter = format(this.cle.tickFormat);
        }
        else {
            // else fallback to default tick formatting settings
            this.cle.tickFormatter = this.xScale.tickFormat(this.cle.ticks || DEFAULT_TICKS, this.cle.tickFormat || DEFAULT_TICK_FORMAT);
        }
    }
}
//# sourceMappingURL=x-scale-axis.js.map